#version 330 core

uniform sampler2D colorTexture;
uniform float gamma;
uniform float exposure;
uniform int useHDR;

in vec2 fragTexCoords;

out vec4 fragColor;

float A = 0.15;
float B = 0.50;
float C = 0.10;
float D = 0.20;
float E = 0.02;
float F = 0.30;
float W = 11.2; //white point

//basics via http://www.learnopengl.com/#!Advanced-Lighting/HDR
//uncharted tone-mapping via http://filmicgames.com/archives/75

vec3 unchartedTonemap(vec3 x){
	return ((x * (A*x + C*B) + D*E) / (x * (A*x + B) + D*F)) - E/F;
}

void main()
{
	vec3 hdrColor = texture(colorTexture, fragTexCoords).rgb;
	vec3 ldrColor;
	
	if(useHDR == 1){
		//ldrColor = hdrColor / (hdrColor + vec3(1.0)); //Reinhard tone mapping
		ldrColor = hdrColor * exposure; //exposure adjustment

		vec3 tonemapped = unchartedTonemap(ldrColor);
		vec3 whiteScale = 1.0/unchartedTonemap(vec3(W));
		ldrColor = tonemapped * whiteScale;

	} else {
		ldrColor = hdrColor;
	}

	ldrColor = pow(ldrColor, vec3(1.0 / gamma)); //gamma correction

	fragColor = vec4(ldrColor, 1.0);
}