#version 330 core

uniform sampler2D gPositionDepth;
uniform sampler2D gNormal;
uniform sampler2D noise;

uniform vec3 kernelSamples[64];
uniform float radius;
uniform float kernelSize;
uniform float occlusionPower;

uniform mat4 projectionMatrix;

uniform vec2 screensize;

in vec2 fragTexCoords;

out float fragColor;

//via http://learnopengl.com/#!Advanced-Lighting/SSAO

vec2 calcTexCoords(){
	return fragTexCoords / screensize;
}
void main()
{
	//vec2 texCoords = calcTexCoords();
	vec2 texCoords = fragTexCoords;

	vec2 noiseScale = vec2(screensize.x/4.0, screensize.y/4.0); //scale-factor for uvs (-> tiling); div screensize by noiseTex size

	vec3 P = texture(gPositionDepth, texCoords).xyz;
	vec3 N = texture(gNormal, texCoords).xyz;
	vec3 random = texture(noise, texCoords * noiseScale).xyz;

	//construct TBN-matrix based on noise
	vec3 T = normalize(random - N * dot(random, N));
	vec3 B = cross(N, T);
	mat3 TBN = mat3(T,B,N);

	float occlusion = 0.0;
	//iterate over kernel samples...
	for(int i = 0; i < kernelSize; ++i){
		vec3 sample = TBN * kernelSamples[i]; //transform them from tangent to viewspace...
		sample = P + sample * radius; //add them to fragment position... (-> offset fragment position inside hemisphere)

		vec4 offset = vec4(sample, 1.0);
		offset = projectionMatrix * offset; //viewspace to clipspace...
		offset.xyz /= offset.w; //perspective division... (-> NDC)
		offset.xyz = offset.xyz * 0.5 + 0.5; //transform to [0.0, 1.0] (to sample depth texture)

		float sampleDepth = -texture(gPositionDepth, offset.xy).w; //depth at sample position from viewer's perspective (first non-occluded visible fragment)

		//if sample's depth is larger than stored depth...
		if(sampleDepth >= sample.z){
			//range-check, so fragments close to an edge will not be tested against surface far behind
			float newRadius = radius / abs(P.z - sampleDepth);
			float rangeCheck = smoothstep(0.0, 1.0, newRadius);
			occlusion = occlusion + rangeCheck;
		}
	}

	//normalize occlusion(-factor)
	float occlusionFactor = 1.0 - (occlusion / kernelSize);

	fragColor = pow(occlusionFactor, occlusionPower);
}